"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.useTransferData = void 0;
const vue_1 = require("vue");
const vooks_1 = require("vooks");
// eslint-disable-next-line @typescript-eslint/explicit-function-return-type
function useTransferData(props) {
    const uncontrolledValueRef = (0, vue_1.ref)(props.defaultValue);
    const mergedValueRef = (0, vooks_1.useMergedState)((0, vue_1.toRef)(props, 'value'), uncontrolledValueRef);
    const optionsMapRef = (0, vue_1.computed)(() => {
        const map = new Map();
        (props.options || []).forEach((opt) => map.set(opt.value, opt));
        return map;
    });
    const targetValueSetRef = (0, vue_1.computed)(() => new Set(mergedValueRef.value || []));
    const targetOptionsRef = (0, vue_1.computed)(() => {
        const optionMap = optionsMapRef.value;
        const targetOptions = [];
        (mergedValueRef.value || []).forEach((v) => {
            const option = optionMap.get(v);
            if (option) {
                targetOptions.push(option);
            }
        });
        return targetOptions;
    });
    const srcPatternRef = (0, vue_1.ref)('');
    const filteredSrcOptionsRef = (0, vue_1.computed)(() => {
        if (!props.filterable)
            return props.options;
        const { filter } = props;
        return props.options.filter((opt) => filter(srcPatternRef.value, opt));
    });
    const mergedValueSetRef = (0, vue_1.computed)(() => {
        const { value } = mergedValueRef;
        if (value === null)
            return new Set();
        return new Set(value);
    });
    const valueSetForSelectAllRef = (0, vue_1.computed)(() => {
        const mergedValueSet = mergedValueSetRef.value;
        const values = [];
        filteredSrcOptionsRef.value.forEach((option) => {
            if (!option.disabled || mergedValueSet.has(option.value)) {
                values.push(option.value);
            }
        });
        return new Set(values);
    });
    const valueSetForUnselectAllRef = (0, vue_1.computed)(() => {
        const values = [];
        targetOptionsRef.value.forEach((option) => {
            if (option.disabled) {
                values.push(option.value);
            }
        });
        return new Set(values);
    });
    const canNotSelectAnythingRef = (0, vue_1.computed)(() => {
        return filteredSrcOptionsRef.value.every((option) => option.disabled);
    });
    const allCheckedRef = (0, vue_1.computed)(() => {
        if (!filteredSrcOptionsRef.value.length) {
            return false;
        }
        const mergedValueSet = mergedValueSetRef.value;
        return filteredSrcOptionsRef.value.every((option) => option.disabled || mergedValueSet.has(option.value));
    });
    const canBeClearedRef = (0, vue_1.computed)(() => {
        return targetOptionsRef.value.some((option) => !option.disabled);
    });
    function handleSrcFilterUpdateValue(value) {
        srcPatternRef.value = value !== null && value !== void 0 ? value : '';
    }
    return {
        uncontrolledValueRef,
        mergedValueRef,
        targetValueSetRef,
        valueSetForSelectAllRef,
        valueSetForUnselectAllRef,
        targetOptionsRef,
        filteredSrcOptionsRef,
        canNotSelectAnythingRef,
        canBeClearedRef,
        allCheckedRef,
        srcPatternRef,
        handleSrcFilterUpdateValue
    };
}
exports.useTransferData = useTransferData;
